package controller;

import java.rmi.RemoteException;
import java.rmi.server.UnicastRemoteObject;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;
import javax.swing.JOptionPane;
import javax.swing.SwingUtilities;

/**
 * Implementierung des Client-Interfaces
 * 
 * @author Thomas Kramer, eMail: th-rzv@gmx.de
 * 
 */
public class ClientInterfaceImpl extends UnicastRemoteObject implements
		ClientInterface {
	private static final long serialVersionUID = 1L;
	
	private UserRole roleType;
	
	private String clientIP = "";	
	private String loginTime = "";

	/**
	 * Konstruktor
	 * 
	 * @throws RemoteException
	 */
	public ClientInterfaceImpl(UserRole roleType) throws RemoteException {
		super();
		
		this.roleType = roleType;  
		
		SimpleDateFormat sdf = new SimpleDateFormat("HH:mm:ss");
		this.loginTime = sdf.format(new Date());		
	}
	
	/**
	 * gibt den eingeloggten Benutzertyp zurck
	 * 
	 * @return UserRole
	 */
	@Override	
	public UserRole getUserRole()
	{
		return this.roleType;
	}
	
	/**
	 * holt die Client-IP-Adresse
	 */
	@Override	
	public String getClientIP()
	{
		return this.clientIP;
	}
	
	/**
	 * setzt die Client-IP-Adresse
	 * 
	 * @param ip
	 */
	@Override	
	public void setClientIP(String ip)
	{
		clientIP = ip;
	}	
	
	/**
	 * holt die Loginzeit
	 */
	@Override	
	public String getLoginTime()
	{
		return this.loginTime;
	}
	
	/**
	 * Gibt einen eindeutigen Kennzeichner des Clients zurck -
	 * bestehend aus Client-IP und Startzeit (Loginzeit)
	 * (IP reicht nicht wenn mit mehreren Instanzen auf demselben Rechner
	 * getestet wird)
	 * 
	 * @return unique identifier
	 */
	@Override	
	public String getUniqueIdentifier()
	{
		return getClientIP() + "," + getLoginTime();		
	}

	/**
	 * Liste von Objekten vom Server empfangen und anwendungsintern an alle
	 * Views weiterreichen
	 */
	@Override	
	public void update(Class t, List<Object> o, ClientInterface dataInitiator) throws RemoteException {
		/**
		 * Zeitstempel ausgeben
		 */
		SimpleDateFormat sdf = new SimpleDateFormat("HH:mm:ss");
		String time = sdf.format(new Date());
		System.out.println("Daten empfangen: " + time);
	}

	/**
	 * Server wurde beendet, entsprechend Clients auch beenden
	 * 
	 */
	@Override
	public void serverTerminated() throws RemoteException {
		/*
		 * Beendigung des Clients soll asynchron zur Beendigung des Servers
		 * laufen
		 */
		SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				JOptionPane
						.showMessageDialog(
								null,
								"Der Server wurde beendet, der Client kann entsprechend auch nicht mehr weiterlaufen!",
								"Fehler", JOptionPane.ERROR_MESSAGE);
				System.err
						.println("Server wurde beendet, entsprechend wird der Client auch beendet!");
				System.exit(1);
			}
		});
	}

	@Override
	public void closeChangeWindow(Object changeObject,
			ClientInterface changeObjectRegisteredByUser)
			throws RemoteException {
		// TODO Auto-generated method stub
		
	}
}