package controller;


import java.rmi.NotBoundException;
import java.rmi.RMISecurityManager;
import java.rmi.RemoteException;
import java.rmi.registry.LocateRegistry;
import java.rmi.registry.Registry;
import javax.swing.JOptionPane;
import javax.swing.SwingUtilities;
import javax.swing.UIManager;

import controller.modelController.ModelController;

/**
 * Controller des MVC-Patterns
 * 
 * @author Thomas Kramer, eMail: th-rzv@gmx.de
 * 
 */
public class Controller {
	private Controller controller = null;
	
	/* Server-Objekte sollten als statisch deklariert werden damit sie nie vom Garbage Collector abgerumt werden */
	public static ServerInterface rmiServer = null;
	public static ClientInterfaceImpl rmiClient = null;
	private ModelController modelController = null;	
	
	/**
	 * Konstruktor
	 * 
	 * @param args
	 */
	public Controller(String[] args) {
		controller = this;
		boolean ok = false;		
		
		/**
		 * benutze natives Windows-Look and Feel, siehe
		 * 
		 * http://www.apl.jhu.edu/~hall/java
		 * /Swing-Tutorial/Swing-Tutorial-LAF.html
		 */
		try {
			UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
		} catch (Exception e) {
			System.out.println("Error setting native LAF: " + e);
		}					
				

		/**
		 * Serververbindung starten
		 */
		String serverPort = String.valueOf(Registry.REGISTRY_PORT);
		//String serverAddress = "192.168.178.45";
		String serverAddress = "localhost";

		System.setProperty("java.security.policy", "file:./client.policy");

		
		if (args.length > 1) {
			serverPort = args[0];
			serverAddress = args[1];
		}

		/**
		 * Erstellen des Security-Managers fr RMI-Kommunikation
		 */
		if (System.getSecurityManager() == null) {
			System.setSecurityManager(new RMISecurityManager());
		}

		try {
			Registry registry = LocateRegistry.getRegistry(serverAddress,
					(new Integer(serverPort)).intValue());
			rmiServer = (ServerInterface) (registry
					.lookup(ServerInterface.SERVICE_NAME));
			ok = true;
		} catch (java.rmi.ConnectException ce) {
			JOptionPane
					.showMessageDialog(
							null,
							"Der Projektverwaltungs-Server wurde noch nicht gestartet!",
							"Fehler", JOptionPane.ERROR_MESSAGE);
			System.err.println("Server-Anwendung wurde noch nicht gestartet!");
		} catch (RemoteException e) {
			System.err.println(e);
		} catch (NotBoundException e) {
			System.err.println(e);
		}

		/**
		 * GUI starten
		 */
		if (ok) {
			SwingUtilities.invokeLater(new Runnable() {
				public void run() {
					try {
						modelController = new ModelController();
						
						rmiClient = new ClientInterfaceImpl(UserRole.Administrator);
						rmiServer.addObserver(rmiClient);						
					} catch (RemoteException e) {
						System.err
								.println("Fehler beim Anmelden zum Server bzw. hinzufgen zur Observer-Liste!");
						System.err.println(e);
						System.err
								.println("Anwendung wird fehlerbedingt beendet!");
						System.exit(1);
					}

					/**
					 * Das LoginFrame selbst mu nicht als Beobachter
					 * hinzugefgt werden
					 */
				}
			});
		} else {
			System.err.println("Anwendung wird fehlerbedingt beendet!");
			System.exit(1);
		}
	}
	
	/**
	 * Registrieren eines Clients beim Server
	 * 
	 * @throws RemoteException Exceptions werden nach oben weitergereicht
	 */
	public static void addServerObserver(UserRole roleType) throws RemoteException
	{
		rmiClient = new ClientInterfaceImpl(roleType);
		rmiServer.addObserver(rmiClient);
	}
	
    /**
     * Abmelden eines Clients beim Server
     */
	public static void removeObserver()
	{
		try {
			Controller.rmiServer.removeObserver(Controller.rmiClient);
		} catch (RemoteException e1) {
			// TODO Auto-generated catch block
			e1.printStackTrace();
		}		
	}
	
	/**
	 * liefert den eigenen PC, der sich beim Server angemeldet hat, zurck.
	 * 
	 * @return ClientInterface eigener PC
	 */
	public static ClientInterface getClientObserver()
	{
		return rmiClient;
	}
	
	/**
	 * liefert das Server-Objekt zurck
	 * 
	 * @return ServerInterface Server-Objekt
	 */
	public static ServerInterface getServerObject()
	{
		return rmiServer;
	}			
}
