package controller.modelController;

import java.rmi.RemoteException;
import java.util.List;
import java.util.Observable;
import java.util.Observer;

import controller.ClientInterface;
import controller.Controller;

public class ModelController extends Observable {		
	/**
	 * Konstruktor
	 */
	public ModelController()
	{
		
	}
	
	/**
	 * Wrapper-Methode um Daten vom Server anzufordern. 
	 * 
	 * @param tableClass Klassentyp
	 * @return           Liste von Datenobjekten
	 */
	public static ResultContent getData(Class tableClass)
	{
		ResultContent result = null;
		try {
			result = new ResultContent(tableClass, Controller.getServerObject()
					.getData(tableClass), Controller.getClientObserver());
		} catch (RemoteException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return result;
	}
	
	/**
	 * Wrapper-Methode um gefilterte Daten vom Server anzufordern.
	 *  
	 * @param tableClass        Klassentyp
	 * @param column            Datenbankspalte auf die die Filterbedingung angewandt werden soll
	 * @param checkingClause    Filterbedingung
	 * @param caseSensitivity   Bercksichtigung der Gro- und Kleinschreibung in der Datenbank
	 * @return zurckgelieferte Liste von Objekten die der Bedingung entsprechen
	 */
	public static ResultContent getDataWithClause(Class tableClass, String column, String checkingClause, boolean caseSensitivity)
	{
		ResultContent result = null;
		try {
			result = new ResultContent(tableClass, Controller.getServerObject().getDataWithClause(tableClass, column,
					checkingClause, caseSensitivity), Controller.getClientObserver());
		} catch (RemoteException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return result;
	}
	
	/**
	 * Wrapper-Methode um gefilterte Daten vom Server anzufordern
	 *  
	 * @param tableClass        Klassentyp
	 * @param column            Datenbankspalte auf die die Filterbedingung angewandt werden soll
	 * @param checkingClause    Filterbedingung
	 * @return zurckgelieferte Liste von Objekten die der Bedingung entsprechen
	 */
	public static ResultContent getDataWithClause(Class tableClass, String column, Integer checkingClause)
	{
		ResultContent result = null;
		try {
			result = new ResultContent(tableClass, Controller.getServerObject().getDataWithClause(tableClass, column,
					checkingClause), Controller.getClientObserver());
		} catch (RemoteException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return result;
	}		
	
	/**
	 * Wrapper-Methode um neue Daten in die Datenbank zu schreiben
	 * 
	 * @param object Datenobjekt
	 * @return true, falls Speichern funktioniert hat
	 */
	public static boolean insertData(Object object)
	{
		boolean result = true;
		if (object != null) {
			try {
				result = Controller.getServerObject().insertData(object, Controller.getClientObserver());
			} catch (RemoteException e) {
				result = false;

				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
		return result;		
	}
	
	/**
	 * Wrapper-Methode um neue Daten in die Datenbank zu schreiben
	 * 
	 * @param object Liste von Datenobjekten
	 * @return true, falls Speichern funktioniert hat
	 */
	public static boolean insertDataCollection(List<Object> object)
	{
		boolean result = true;
		if (object != null) {
			try {
				result = Controller.getServerObject().insertDataCollection(object, Controller.getClientObserver());
			} catch (RemoteException e) {
				result = false;

				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
		return result;		
	}	
	
	/**
	 * Wrapper-Methode um genderte Daten in die Datenbank zu schreiben
	 * 
	 * @param object Datenobjekt
	 * @return true, falls Speichern funktioniert hat
	 */
	public static boolean changeData(Object object)
	{
		boolean result = true;
		if (object != null) {
			try {
				result = Controller.getServerObject().changeData(object, Controller.getClientObserver());
			} catch (RemoteException e) {
				result = false;

				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
		return result;		
	}	
	
	/**
	 * Registrieren eines ChangeObjects beim Server, damit es kein anderer Benutzer (auer einem Admin)
	 * bearbeiten kann. 
	 * 
	 * @param changeObject   changeObject
	 */
	public static void registerChangeObject(Object changeObject) 
	{
		if (changeObject != null) {
			try {
				Controller.getServerObject().registerChangeObject(changeObject, Controller.getClientObserver());
			} catch (RemoteException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
		
	}

	/**
	 * Deregistrieren eines ChangeObjects beim Server. 
	 * 
	 * @param changeObject   changeObject
	 */	
	public static void unregisterChangeObject(Object changeObject) 
	{
		if (changeObject != null) {
			try {
				Controller.getServerObject().unregisterChangeObject(changeObject, Controller.getClientObserver());
			} catch (RemoteException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}		
	}
	
	/**
	 * Nachschauen ob das ChangeObject im Server schon als "in Bearbeitung" registriert wurde, wenn ja
	 * Zugriff untersagen.
	 * 
	 * Eine Ausnahme stellt der Admin dar, dieser bekommt immer Zugriff.
	 * 
	 * @param changeObject   changeObject
	 * @return true, wenn Zugriff aufs Datenobjekt erlaubt
	 */
	public static boolean accessToChangeObjectAllowed(Object changeObject) 
	{
		boolean result = true;
		if (changeObject != null) {
			try {
				result = Controller.getServerObject().accessToChangeObjectAllowed(changeObject, Controller.getClientObserver());
			} catch (RemoteException e) {
				result = false;

				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
		return result;				
	}
	
	/**
	 * Wrapper-Methode um ein Daten-Objekt in der Datenbank zu lschen
	 * 
	 * @param activeDataObjects Datenobjekt
	 * @return true, falls Aktion geglckt
	 */
	public static boolean deleteData(Object object)
	{
		boolean result = true;
		if (object != null) {
			try {
				result = Controller.getServerObject().deleteData(object, Controller.getClientObserver());
			} catch (RemoteException e) {
				result = false;
				
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
		return result;
	}	
	
	/**
	 * Objekte vom Server bers Observer-Pattern (Netzwerk) empfangen, anwendungsintern
	 * Datenobjekte aktualisieren und Views aktualisieren
	 * 
	 * @param object
	 *            Daten-Objekt
	 */
	public void delegateObjects(Class t, List<Object> object, ClientInterface dataInitiator) {	
		tellChangedData(t, object, dataInitiator);
	}
	
	/**
	 * fgt ein Formular- oder Dialogobjekt der anwendungsinternen Observerliste hinzu
	 * 
	 * @param o Observer
	 */
	public void addObserverObject(Observer o)
	{
		addObserver(o);		
	}

	/**
	 * 
	 * Daten weiterreichen, gekapselt in ein ResultContent-Objekt
	 * 
	 * @param results
	 *            Liste mit eventuellen Ergebnissen
	 */
	private void tellChangedData(Class t, List<Object> results, ClientInterface dataInitiator) {
		if (t != null) {
			submitDataToViews(new ResultContent(t, results, dataInitiator));
		} else {
			submitDataToViews(null);
		}
	}

	/**
	 * benachrichtige Views, dass Daten gendert wurden.
	 * 
	 * @param object
	 *            gendertes Objekt
	 */
	private void submitDataToViews(final Object object) {
		if (object != null)
		{

		}
	}			
}
