package controller.modelController;

import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.List;

import controller.ClientInterface;


/**
 * Klasse zum Kapseln beliebiger Daten die von EclipseLink ausgelesen wurden
 * 
 * @author Thomas Kramer, eMail: th-rzv@gmx.de
 * 
 */
public class ResultContent {
	private Class listType = null;
	private List<Object> listContent = null;
	private ClientInterface dataInitiator = null;	

	private Date creationTime;

	/**
	 * Konstruktor
	 * 
	 * @param listType
	 *            Datentyp der enthaltenen Daten
	 * @param listContent
	 *            Daten
	 * @param dataInitiator der Client der veranlasst hat dass Datennderungen bers Observer-Pattern angeliefert werden
	 */
	public ResultContent(Class listType, List<Object> listContent, ClientInterface dataInitiator) {
		this.listType = listType;
		this.listContent = listContent;
		this.dataInitiator = dataInitiator;
		
		creationTime = new Date();	
	}
	
	/**
	 * liefert Uhrzeit und Datum zurck, wann das ResultContent-Objekt instanziiert wurde
	 * 
	 * @return Date-Object
	 */
	public Date getCreationTime()
	{
		return creationTime;
	}
	
	/**
	 * liefert Uhrzeit und Datum als String zurck, wann das ResultContent-Objekt instanziiert wurde
	 * 
	 * @return Datum und Uhrzeit als String
	 */	
	public String getCreationTimeAsString()
	{
		/* http://javawiki.sowas.com/doku.php?id=java:date-format */
		SimpleDateFormat format = new SimpleDateFormat("dd.MM.yyyy HH:mm:ss");
		return format.format(creationTime);				
	}	

	/**
	 * Zurckliefern des Typs der gespeicherten Daten
	 * 
	 * @return Object
	 */
	public Class getListType() {
		return listType;
	}

	/**
	 * Zurckliefern der eigentlichen Daten
	 * 
	 * @return List<Object>
	 */
	public List<Object> getListContent() {
		return listContent;
	}
	
	/**
	 * Zurckliefern des Daten-Initiators
	 * 
	 * @return ClientInterface Daten-Initiator
	 */
	public ClientInterface getDataInitiator() {
		return dataInitiator;
	}	

	/**
	 * Auslesen ob Daten enthalten sind
	 * 
	 * @return
	 */
	public boolean isEmpty() {
		return (listType == null || listContent == null || listContent.size() == 0);
	}
}