package databaseModel;

import javax.persistence.*;
import javax.validation.constraints.Min; 
import javax.validation.constraints.NotNull;
import javax.validation.constraints.Pattern;

import org.hibernate.validator.constraints.Length;

import java.util.List;


/**
 * The persistent class for the ANSPRECHPARTNER database table.
 * 
 */
@Entity
public class Ansprechpartner extends AbstractEntity {
	private static final long serialVersionUID = 1L;
	private int id;	
	
	@Length(max = 30, message="Die EMail-Adresse darf maximal 30 Zeichen lang sein")
	@Pattern(regexp="^[_A-Za-z0-9-\\+]+(\\.[_A-Za-z0-9-]+)*@"
			+ "[A-Za-z0-9-]+(\\.[A-Za-z0-9]+)*(\\.[A-Za-z]{2,})$", message="Die EMail-Adresse ist ungueltig")
	@NotNull
	private String email;
	@Length(max = 30, message="Der Name darf maximal 30 Zeichen lang sein")
	@NotNull	
	private String name;
	@NotNull	
	private Unternehmen unternehmenBean;
	@NotNull	
	private List<Projekt> projekts;

	public Ansprechpartner() {
	}	

	@Id	
	@GeneratedValue(strategy=GenerationType.IDENTITY)
	public int getId() {
		return this.id;
	}

	public void setId(int id) {
		this.id = id;
	}

	public String getEmail() {
		return this.email;
	}

	public void setEmail(String email) {
		this.email = email;
	}

	public String getName() {
		return this.name;
	}

	public void setName(String name) {
		this.name = name;
	}


	//bi-directional many-to-one association to Unternehmen
	@ManyToOne
	@JoinColumn(name="UNTERNEHMEN")
	public Unternehmen getUnternehmenBean() {
		return this.unternehmenBean;
	}

	public void setUnternehmenBean(Unternehmen unternehmenBean) {
		this.unternehmenBean = unternehmenBean;
	}


	//bi-directional many-to-one association to Projekt
	@OneToMany(mappedBy="ansprechpartnerBean")	
	public List<Projekt> getProjekts() {
		return this.projekts;
	}

	public void setProjekts(List<Projekt> projekts) {
		this.projekts = projekts;
	}

	public static String getDefaultSortOrder()
	{
		return "name";
	}

	private int version;	
	@Version
	public int getVersion() {
		return this.version;
	}

	public void setVersion(int version) {
		this.version = version;
	}	
}