package databaseModel;

import javax.persistence.*;
import javax.validation.constraints.NotNull;
import javax.validation.constraints.Pattern;

import org.hibernate.validator.constraints.Length;

import java.util.List;


/**
 * The persistent class for the STUDENT database table.
 * 
 */
@Entity
public class Student extends AbstractEntity {
	private static final long serialVersionUID = 1L;
	private int id;
	
	@Length(max = 30, message="Die EMail-Adresse darf maximal 30 Zeichen lang sein")
	@Pattern(regexp="^[_A-Za-z0-9-\\+]+(\\.[_A-Za-z0-9-]+)*@"
			+ "[A-Za-z0-9-]+(\\.[A-Za-z0-9]+)*(\\.[A-Za-z]{2,})$", message="Die EMail-Adresse ist ungueltig")
	@NotNull		
	private String email;
	@NotNull		
	private int mat;
	@Length(max = 30, message="Der Name darf maximal 30 Zeichen lang sein")
	@NotNull		
	private String name;
	private List<Projekt> projekts1;
	private List<Projekt> projekts2;
	private List<Projekt> projekts3;

	public Student() {
	}

	@Id	
	@GeneratedValue(strategy=GenerationType.IDENTITY)
	public int getId() {
		return this.id;
	}

	public void setId(int id) {
		this.id = id;
	}


	public String getEmail() {
		return this.email;
	}

	public void setEmail(String email) {
		this.email = email;
	}


	public int getMat() {
		return this.mat;
	}

	public void setMat(int mat) {
		this.mat = mat;
	}

	public String getName() {
		return this.name;
	}

	public void setName(String name) {
		this.name = name;
	}


	//bi-directional many-to-one association to Projekt
	@OneToMany(mappedBy="student1Bean")
	public List<Projekt> getProjekts1() {
		return this.projekts1;
	}

	public void setProjekts1(List<Projekt> projekts1) {
		this.projekts1 = projekts1;
	}


	//bi-directional many-to-one association to Projekt
	@OneToMany(mappedBy="student2Bean")
	public List<Projekt> getProjekts2() {
		return this.projekts2;
	}

	public void setProjekts2(List<Projekt> projekts2) {
		this.projekts2 = projekts2;
	}


	//bi-directional many-to-one association to Projekt
	@OneToMany(mappedBy="student3Bean")
	public List<Projekt> getProjekts3() {
		return this.projekts3;
	}

	public void setProjekts3(List<Projekt> projekts3) {
		this.projekts3 = projekts3;
	}
	
	public static String getDefaultSortOrder()
	{
		return "name";
	}
	
	private int version;	
	@Version
	public int getVersion() {
		return this.version;
	}

	public void setVersion(int version) {
		this.version = version;
	}		
}